-- Maybe this works?

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.user_pkg.all;

entity ir_decode_logic is
  port(
    -- general signals
    clk : in std_logic;
    rst : in std_logic;

    -- vJTAG signals
    tck   : in  std_logic;
    ir_in : in  std_logic_vector(IR_LENGTH-1 downto 0);
    cdr   : in  std_logic;
    sdr   : in  std_logic;
    udr   : in  std_logic;
    tdi   : in  std_logic;
    tdo   : out std_logic;

    -- I/O signals
    mmap_data     : out std_logic_vector(MMAP_DATA_RANGE);
    mmap_addr     : out std_logic_vector(MMAP_ADDR_RANGE);
    mmap_wr_rd    : out std_logic;
    mmap_valid    : out std_logic;
    mmap_data_out : in  std_logic_vector(MMAP_DATA_RANGE));

end ir_decode_logic;

architecture BHV of ir_decode_logic is
  -- If changing the VDR widths, be sure to update the TCL files
  signal addr_reg   : std_logic_vector(31 downto 0);
  signal data_reg   : std_logic_vector(31 downto 0);
  signal wr_rd_reg  : std_logic;
  signal bypass_reg : std_logic;
begin

  -----------------------------------------------------------------
  -- vJTAG Clock Domain
  -----------------------------------------------------------------
  process(tck, rst)
  begin
    if (rst = '1') then
      addr_reg   <= (others => '0');
      data_reg   <= (others => '0');
      wr_rd_reg  <= '0';
      bypass_reg <= '0';
    elsif (rising_edge(tck)) then
      bypass_reg <= tdi;

      -- Only write to memory map when in correct IR code
      case ir_in is
        when C_WR_DATA_IR =>
          wr_rd_reg <= '1';
        when others =>
          wr_rd_reg <= '0';
      end case;

      -- vJTAG shifting in data from tdi
      if (sdr = '1') then
        case ir_in is
          when C_ADDR_IR =>
            addr_reg <= tdi & addr_reg(addr_reg'left downto 1);
          when C_RD_DATA_IR =>
            data_reg <= tdi & data_reg(data_reg'left downto 1);
          when c_WR_DATA_IR =>
            data_reg <= tdi & data_reg(data_reg'left downto 1);
          when others =>
            null;
        end case;
      end if;

      -- vJTAG capturing data from mmap
      if (cdr = '1') then
        case ir_in is
          when C_RD_DATA_IR =>
            data_reg                  <= (others => '0');
            data_reg(MMAP_DATA_RANGE) <= mmap_data_out;
          when others =>
            null;
        end case;
      end if;
    end if;
  end process;

  -- Maintain tdo continuity
  tdo <= data_reg(0) when ir_in = C_RD_DATA_IR else bypass_reg;

  -----------------------------------------------------------------
  -- User Clock Domain
  -----------------------------------------------------------------
  -- Handshake
  U_HANDSHAKE : entity work.handshake
    port map (
      clk_src    => tck,
      clk_dest   => clk,
      rst        => rst,
      start_pl   => udr,
      take_it_pl => mmap_valid,
      got_it_pl  => open  --shouldn't need it given vJTAG operation
      );

  -- vJTAG's udr indicates outputs are stable. Handshake is used to generate valid pulse
  mmap_addr  <= addr_reg(MMAP_ADDR_RANGE);
  mmap_data  <= data_reg(MMAP_DATA_RANGE);
  mmap_wr_rd <= wr_rd_reg;
  -- Latch mmap signals. Probably not need as long as mmap_valid is handshaked
  --process(clk, rst)
  --begin
  --    if (rst = '1') then
  --            mmap_addr       <= (others => '0');
  --            mmap_data       <= (others => '0');
  --            mmap_wr_rd      <= '0';
  --            mmap_valid      <= '0';
  --    elsif (rising_edge(clk)) then
  --            mmap_wr_rd      <= '0'; -- ensure only wr pulses
  --            if (stable_output = '1') then
  --                    mmap_addr       <= addr_reg;
  --                    mmap_data       <= data_reg;
  --                    mmap_wr_rd      <= wr_rd_reg;
  --            end if;
  --            mmap_valid <= stable_output;
  --    end if;
  --end process;

end architecture;


