-- Greg Stitt
-- Jayson Salkey
-- Abdelali Nouina
-- David Wilson
-- University of Florida

library ieee;
use ieee.std_logic_1164.all;
use work.user_pkg.all;

entity jtag_wrapper is
  port(
    clk    : in  std_logic;
    rst    : in  std_logic;
    go     : out std_logic;
    input1 : out std_logic_vector(DATA_WIDTH-1 downto 0);
    input2 : out std_logic_vector(DATA_WIDTH-1 downto 0);
    result : in  std_logic_vector(DATA_WIDTH-1 downto 0);
    done   : in  std_logic
    );
end jtag_wrapper;

architecture bhvr of jtag_wrapper is
  -- Jtag signals
  signal tck   : std_logic;
  signal tdo   : std_logic;
  signal tdi   : std_logic;
  signal ir_in : std_logic_vector(IR_LENGTH-1 downto 0);
  signal sdr   : std_logic;
  signal cdr   : std_logic;
  signal udr   : std_logic;

  -- Memory map signals
  signal mmap_data     : std_logic_vector(MMAP_DATA_RANGE);
  signal mmap_addr     : std_logic_vector(MMAP_ADDR_RANGE);
  signal mmap_rd_wr    : std_logic;
  signal mmap_valid    : std_logic;
  signal mmap_data_out : std_logic_vector(MMAP_DATA_RANGE);
begin

  --vJTAG Megafunction call
  U_vJTAG : entity work.vJTAG
    port map(
      tck               => tck,
      tdo               => tdo,         --in
      tdi               => tdi,         --out
      ir_out            => "000",
      ir_in             => ir_in,
      virtual_state_sdr => sdr,
      virtual_state_udr => udr,
      virtual_state_cdr => cdr
      );

  --Decode IR commands and generate appropriate mmap signals
  U_IR_DECODE : entity work.ir_decode_logic
    port map(
      tck           => tck,
      clk           => clk,
      rst           => rst,
      ir_in         => ir_in,
      udr           => udr,
      cdr           => cdr,
      sdr           => sdr,
      tdi           => tdi,
      tdo           => tdo,
      mmap_data     => mmap_data,
      mmap_addr     => mmap_addr,
      mmap_wr_rd    => mmap_rd_wr,
      mmap_valid    => mmap_valid,
      mmap_data_out => mmap_data_out
      );

  --Memory map for user_app
  U_MEMORY_MAP : entity work.memory_map
    port map(
      clk          => clk,
      rst          => rst,
      valid        => mmap_valid,
      rd_wr        => mmap_rd_wr,
      addr         => mmap_addr,
      data         => mmap_data,
      go           => go,
      input1       => input1,
      input2       => input2,
      result       => result,
      result_to_sw => mmap_data_out,
      done         => done
      );
end architecture;
