-- Greg Stitt
-- Jayson Salkey
-- Abdelali Nouina
-- David Wilson
-- University of Florida

-- Entity: memory_map
-- This entity establishes connections with user-defined addresses and
-- internal FPGA components (e.g. registers and blockRAMs).
--
-- Note: Make sure to add any new addresses to user_pkg. Also, in your C code,
-- make sure to use the same constants.

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.user_pkg.all;

entity memory_map is
  port (
    clk          : in  std_logic;
    rst          : in  std_logic;
    valid        : in  std_logic;
    rd_wr        : in  std_logic;
    addr         : in  std_logic_vector(MMAP_ADDR_RANGE);
    data         : in  std_logic_vector(MMAP_DATA_RANGE);
    input1       : out std_logic_vector(MMAP_DATA_RANGE);
    input2       : out std_logic_vector(MMAP_DATA_RANGE);
    result       : in  std_logic_vector(MMAP_DATA_RANGE);
    result_to_sw : out std_logic_vector(MMAP_DATA_RANGE);
    done         : in  std_logic;
    go           : out std_logic
    );
end memory_map;

architecture BHV of memory_map is

  signal input1_reg : std_logic_vector(MMAP_DATA_RANGE);
  signal input2_reg : std_logic_vector(MMAP_DATA_RANGE);
  signal go_reg     : std_logic;
  signal done_reg   : std_logic;
  signal rd_data    : std_logic_vector(MMAP_DATA_RANGE);

begin
  -- memory map writes (i.e. SW->FPGA)
  process (clk, rst)
  begin
    if (rst = '1') then
      go_reg     <= '0';
      input1_reg <= (others => '0');
      input2_reg <= (others => '0');

    elsif (rising_edge(clk)) then
      if (valid = '1') then

        -- writes
        if (rd_wr = '1') then
          case to_integer(unsigned(addr)) is
            when C_GO_ADDR =>
              go_reg <= data(0);

            when In1_ADDR =>
              input1_reg <= data;

            when In2_ADDR =>
              input2_reg <= data;

            when others =>
              null;
          end case;
        end if;

        -- reads
        if (rd_wr = '0')then
          case to_integer(unsigned(addr)) is

            when RESULT_ADDR =>
              rd_data <= result;

            when DONE_ADDR =>
              rd_data(DATA_WIDTH-1 downto 1) <= (others => '0');
              rd_data(0)                     <= done;

            when others =>
              null;
          end case;
        end if;
      end if;
    end if;
  end process;

  go           <= go_reg;
  input1       <= input1_reg;
  input2       <= input2_reg;
  result_to_sw <= rd_data;

end BHV;
