-- Greg Stitt
-- Jayson Salkey
-- Abdelali Nouina
-- David Wilson
-- University of Florida

-- The following entity is the top-level entity for lab 5 extra credit. No 
-- changes are required, but you need to map the I/O to the appropriate pins 
-- on the board.

-- I/O Explanation (assumes the switches are on side of the
--                  board that is closest to you)
-- switch(9) is the leftmost switch
-- button(1) is the top button
-- led5 is the leftmost 7-segment LED
-- ledx_dp is the decimal point on the 7-segment LED for LED x

-- Note: this code will cause a harmless synthesis warning because not all
-- the buttons are used and because some output pins are always '0' or '1'


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.user_pkg.all;

entity top_level is
  port(
    clk50MHz : in  std_logic;
    not_rst  : in  std_logic;
    led0     : out std_logic_vector(6 downto 0);
    led0_dp  : out std_logic;
    led1     : out std_logic_vector(6 downto 0);
    led1_dp  : out std_logic;
    led2     : out std_logic_vector(6 downto 0);
    led2_dp  : out std_logic;
    led3     : out std_logic_vector(6 downto 0);
    led3_dp  : out std_logic
    );
end entity;


architecture str of top_level is
  signal rst : std_logic;

  -- user app signals
  signal done   : std_logic;
  signal go     : std_logic;
  signal result : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal input1 : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal input2 : std_logic_vector(DATA_WIDTH-1 downto 0);

begin

  rst <= not not_rst; -- Buttons are active low
  
  -- Jtag interface
  U_jtag_wrapper : entity work.jtag_wrapper
    port map(
      clk    => clk50MHz,
      rst    => rst,
      go     => go,
      input1 => input1,
      input2 => input2,
      result => result,
      done   => done
      );

  -- User App
  U_GCD : entity work.gcd(FSMD2)
    generic map(
      width => DATA_WIDTH)
    port map (
      clk    => clk50MHz,
      rst    => rst,
      go     => go,
      x      => input1,
      y      => input2,
      output => result,
      done   => done
      );

  -- Debug LEDs
  U_LED_0 : entity work.decoder7seg
    port map(
      input  => result(3 downto 0),
      output => led0);


  U_LED_1 : entity work.decoder7seg
    port map(
      input  => result(7 downto 4),
      output => led1);


  U_LED_2 : entity work.decoder7seg
    port map(
      input  => result(11 downto 8),
      output => led2);

  U_LED_3 : entity work.decoder7seg
    port map(
      input  => '0' & result(14 downto 12),
      output => led3);

  led3_dp <= '1';
  led2_dp <= '1';
  led1_dp <= '1';
  led0_dp <= '1';

end str;
